#!/bin/bash
# Log an error and exit.
# Args:
#   $1 Message to log with the error
#   $2 The error code to return
#   $3 The number of stack frames to skip when printing.

migration_VERBOSE="${migration_VERBOSE:-2}"
  Timestamp=$(date +"[%m%d %H:%M:%S]")
  Info=$(echo -e "[\033[32mInfo\033[0m]")

migration::log::errexit() {
  local err="${PIPESTATUS[*]}"

  # If the shell we are in doesn't have errexit set (common in subshells) then
  # don't dump stacks.
  set +o | grep -qe "-o errexit" || return

  set +o xtrace
  local code="${1:-1}"
  # Print out the stack trace described by $function_stack
  if [ ${#FUNCNAME[@]} -gt 2 ]
  then
    migration::log::error "Call tree:"
    for ((i=1;i<${#FUNCNAME[@]}-1;i++))
    do
      migration::log::error " ${i}: ${BASH_SOURCE[${i}+1]}:${BASH_LINENO[${i}]} ${FUNCNAME[${i}]}(...)"
    done
  fi
  migration::log::error_exit "Error in ${BASH_SOURCE[1]}:${BASH_LINENO[0]}. '${BASH_COMMAND}' exited with status ${err}" "${1:-1}" 1
}

migration::log::error_exit() {
  local message="${1:-}"
  local code="${2:-1}"
  local stack_skip="${3:-0}"
  stack_skip=$((stack_skip + 1))

  if [[ ${migration_VERBOSE} -ge 4 ]]; then
    local source_file=${BASH_SOURCE[${stack_skip}]}
    local source_line=${BASH_LINENO[$((stack_skip - 1))]}
    echo "!!! Error in ${source_file}:${source_line}" >&2
    [[ -z ${1-} ]] || {
      echo "  ${1}" >&2
    }

    migration::log::stack ${stack_skip}

    echo "Exiting with status ${code}" >&2
  fi

  exit "${code}"
}

migration::log::stack() {
  local stack_skip=${1:-0}
  stack_skip=$((stack_skip + 1))
  if [[ ${#FUNCNAME[@]} -gt ${stack_skip} ]]; then
    echo "Call stack:" >&2
    local i
    for ((i=1 ; i <= ${#FUNCNAME[@]} - stack_skip ; i++))
    do
      local frame_no=$((i - 1 + stack_skip))
      local source_file=${BASH_SOURCE[${frame_no}]}
      local source_lineno=${BASH_LINENO[$((frame_no - 1))]}
      local funcname=${FUNCNAME[${frame_no}]}
      echo "  ${i}: ${source_file}:${source_lineno} ${funcname}(...)" >&2
    done
  fi
}


migration::log::error() {
  timestamp=$(date +"[%m%d %H:%M:%S]")
  echo "!!! ${timestamp} ${1-}" >&2
  shift
  for message; do
    echo "    ${message}" >&2
  done
}

migration::log::error() {
  timestamp=$(date +"[%m%d %H:%M:%S]")
  echo -e "[\033[31mError\033[0m]${timestamp} ${1-}" >&2
  shift
  for message; do
    echo "    ${message}" >&2
  done
}

migration::log::info() {
  local V="${V:-0}"
  if [[ ${migration_VERBOSE} < ${V} ]]; then
    return
  fi
  timestamp=$(date +"[%m%d %H:%M:%S]")
  echo -e "[\033[32mInfo\033[0m]${timestamp} ${1-}" >&2
  shift
  for message; do
    echo "    ${message}" >&2
  done
    # echo -n $info
    # echo -n $timestamp
    # echo -e "[\033[32mInfo\033[0m]${timestamp} ${message}"
    # new_msg=${info}_${timestamp}_${message}
    # echo "${timestamp}" 
    # if [[ $(echo ${message}|wc -c) -gt 24 ]];then
    # echo "$(echo $new_msg|wc -c)"
#   for message; do

#     echo -n "$Info ${message}"

#   done

}

migration::log::status() {
    timestamp=$(date +"[%m%d %H:%M:%S]")
  for message; do
    echo -e  "[\033[32mInfo\033[0m]${timestamp} ${message}"
  done
}


